; SLIDE SHOW SAMPLE - DAN1 FORMAT
; COLECOVISION, 2016
;
; Idea by Paulo Silva, Portugal
; Coded by Daniel Bienvenu, Canada
; DAN1 compression by Daniel Bienvenu, Canada
;
; COMPILE WITH TNIASM
; 	> TNIASM filename.asm

fname "sshowdan1opt.rom"

; CONSTANTS

NMI_FLAG:	equ	$7000

; PROGRAM

	cpu	z80
	
	; COLECO ROM CARTRIDGE $8000-$FFFF
	org	$8000
	
	; COLECO ROM HEADER
	dw	$55AA

	; COLECO BIOS RESERVED RAM ADDRESSES
	dw	0,0,0,0
	
	; POINTER TO ROM CODE ENTRY POINT
	dw	start
	
	; RST ROUTINES
	; RST 08
	ret
	nop
	nop
	; RST 10
	ret
	nop
	nop
	; RST 18
	ret
	nop
	nop
	; RST 20
	ret
	nop
	nop
	; RST 28
	ret
	nop
	nop
	; RST 30
	ret
	nop
	nop
	; RST 38 ; SPINNER
	ret
	nop
	nop	
	
	; JUMP TO NON MASKABLE ROUTINE
	jp	nmi
	
	; COLECO SCREEN DATA : TITLE/AUTHOR/YEAR
	db "SLIDE SHOW SAMPLE IN DAN1/PRESENTS/2016"

	; NON MASKABLE INTERUPT
nmi:
	push	af
	ld		a, 1
	ld		(NMI_FLAG),a	; SET NMI FLAG
	call		$1FDC		; in a, ($BF)
	pop		af
	ret

start:
	; INTERUPT MODE = RST $38
	im		1
	; DISABLE INTETUPT
	di
	
	; CLEAR RAM 7000-73B7
	xor		a
	ld		bc, $03B8
	ld		hl, $7000
	ld		de, $7001
	ld		(hl), a
	ldir
	
	; TURN SOUND OFF
	call		$1FD6

	; CLEAR VRAM 0000-3FFF
	ld		de, $4000
	xor		a
	ld		l,a
	ld		h,a
	call		$1f82
	
	; SCREEN OFF, NMI OFF
	ld		bc, $0182		; vdp_out(1,$82)
	call		$1FD9
	; SET GRAPHIC MODE II
	ld		bc, $0002		; vdp_out(0,2) ; MODE 2
	call		$1FD9
	ld		a, 2
	ld		hl, $1800
	call		$1FB8		; vdp_out(2,ADDR($1800)) ; NAME $1800
	ld		bc, $03FF         ; vdp_out(3,$ff) ; PATTERN $0000
	call		$1FD9
	ld		bc, $0403         ; vdp_out(4,$03) ; COLOR $2000
	call		$1FD9
	ld		de, $1800
	call		default_name_table

	; SLiDE SHOW LOOP
slideshow:
	ld		hl, awblp2
	push	hl
	ld		hl, awblp1
	call		slide
	ld		hl, f1spp2
	push	hl
	ld		hl, f1spp1
	call		slide
	ld		hl, h6exp2
	push	hl
	ld		hl, h6exp1
	call		slide
	ld		hl, mgfap2
	push	hl
	ld		hl, mgfap1
	call		slide
	ld		hl, sotbp2
	push	hl
	ld		hl, sotbp1
	call		slide
	; TODO : SLIDE SHOW
	jp slideshow

slide:
	call		nmioff
	ld		de, $0000
	call		dan1
	pop		bc
	pop		hl
	push	bc
	ld		de, $2000
	call		dan1
	call		nmion
	ld		b, 100 ; (approx 2 seconds)
	call		delay
	jp		clear

;	DE = NAME TABLE VRAM ADDRESS
default_name_table:
	call		vdpwrite
	ld		d, 3
default_name_table_1:
	xor		a
default_name_table_2:
	out		($BE), a
	nop
	inc		a
	jp		nz,default_name_table_2
	dec		d
	jp		nz,default_name_table_1
	ret

; VDPWRITE
;	INPUT
;		DE = VRAM ADDRESS
;	DESTROY
;		C
vdpwrite:
	ld		c, $BF
	out		(c), e
	set		6, d
	out		(c), d
	res		6, d
	ret
	
nmioff:
	ld		bc, $01C2
	jp		$1FD9

nmion:
	ld		bc, $01E2
	jp		$1FD9

; DELAY
;	INPUT
;		B = DELAY TIME IN SCREEN REFRESH CYCLES
;	DESTROY
;		A, B
delay:
	xor		a
	ld		(NMI_FLAG),a
delay_loop:
	ld		a,(NMI_FLAG)
	or		a
	jr		z, delay_loop
	djnz		delay

clear:
	xor		a
	ld		hl, $2000 ; COLOR VRAM ADDR
	ld		de, $1800 ; SIZE
	call		$1f82 ; FILL_VRAM
	ld		hl, $0000 ; PATTERN VRAM ADDR
	ld		de, $1800 ; SIZE
	jp		$1f82 ; FILL_VRAM
	
; DAN1 - DECOMPRESSION ROUTINE
dan1:
	; Set Write in VRAM at DE
	call		vdpwrite	
	ld		a, $80

; Copy literal byte
dan1_copy_byte:
	ld		b,$01
	jr		dan1_literals2mainloop
	
dan1_special:
	pop		de
	call		getbit
	ret		nc		; EXIT
	ld		b, $1A
	call		dan1_literals
	ld		b, (hl)                 ; load counter value (8 bits)
	inc		b
dan1_literals2mainloop:	
	call		dan1_literals

; Main loop
dan1_main_loop:
	call		getbit                    ; check next bit
	jr		c,dan1_copy_byte
	
; Elias gamma decoding + Special marker
        push	de
        ld		de, $0001
        ld		b,d	
dan1_eliasgamma_0:
        inc		b
	call		getbit			; check next bit
	jr		c, dan1_eliasgamma_value
	bit		4,b
        jr		nz, dan1_special	; special marker "0000000000000000"
	jr		dan1_eliasgamma_0
dan1_eliasgamma_value_loop:
        call		getbite			; check next bit -> DE
        rl		d
dan1_eliasgamma_value:
	djnz		dan1_eliasgamma_value_loop
	push	de
	pop		bc			; BC = LENGTH
	
; Get Offset value
	ld		d, $00		; Reset Offset to #0x0000
	
; ON LEN GOTO TWO_OFFSETS, THREE_OFFSETS
; GOTO FOUR_OFFSETS

	ex		af,af'
	ld		a,b
	or		a
	jr		z, dan1_bzero
	ld		a, $03
dan1_bzero:
	or		c
	ld		e, a
	ex		af, af'
	dec		e
	jr		z, dan1_offset2
	dec		e
	jr		z, dan1_offset3
	ld		e, d

dan1_offset4:
	call		getbit			; check next bit
	jr		nc, dan1_offset3
        call		getnibblee			; get next nibble -> E
	inc		e
	ld		d,e				; D = E+1
	jr		dan1_offset3a
dan1_offset3:
	call		getbit			; check next bit
	jr		nc, dan1_offset2
dan1_offset3a:
        ld     	e, (hl)			; load offset offset value (8 bits)
        inc   	hl
	ex		af, af'
	ld		a,e
	add		a, $12
	ld		e,a
	jr		nc, dan1_offset3b
	inc		d
dan1_offset3b:
	ex		af,af'
	jr		dan1_copy_from_offset
dan1_offset2:
	call		getbit			; check next bit
	jr      	nc, dan1_offset1
        call		getnibblee			; get next nibble -> E
	inc		e
	inc		e
	jr		dan1_copy_from_offset
dan1_offset1:
	call		getbit			; check next bit
	rl		e
	
; Copy previously seen bytes
dan1_copy_from_offset:
        ex		(sp), hl                ; store source, restore destination
        push	hl                      ; store destination
	scf
        sbc		hl, de                  ; HL = source = destination - offset - 1
        pop		de                      ; DE = destination
	; BC = count
	; COPY BYTES
	ex		af,af'
	set		6,d
dan1_copybytes_loop:
	push	bc
	ld		c, $BF
	out		(c), l
	nop
	out		(c), h
	inc		hl
	nop
	nop
	in		a, ($BE)
	nop
	nop
	nop
	out		(c), e
	nop
	out		(c), d
	inc		de
	nop
	nop
	out		($BE), a
	pop		bc
	dec		bc
	ld		a,b
	or		c
	jr		nz, dan1_copybytes_loop
	res		6,d
	ex		af,af'
        pop		hl		; restore source address (compressed data)
        jp		dan1_main_loop
	
dan1_literals:
	ld		c, $BE
dan1_literals_loop:
	outi
	inc		de
	jr		nz, dan1_literals_loop
	ret

getnibblee:
        call	getbite	; get next bit -> E
        call	getbite	; get next bit -> E
        call	getbite	; get next bit -> E
getbite:
        call	getbit	; get next bit
        rl      e		; push bit into E
	ret

getbit:
	add		a, a
  	ret		nz
	ld		a, (hl)
	inc		hl
	rla
	ret

; DATA

awblp1:
	incbin	"awblp1.dan1"
awblp2:
	incbin	"awblp2.dan1"
f1spp1:
	incbin	"f1spp1.dan1"
f1spp2:
	incbin	"f1spp2.dan1"
h6exp1:
	incbin	"h6exp1.dan1"
h6exp2:
	incbin	"h6exp2.dan1"
mgfap1:
	incbin	"mgfap1.dan1"
mgfap2:
	incbin	"mgfap2.dan1"
sotbp1:
	incbin	"sotbp1.dan1"
sotbp2:
	incbin	"sotbp2.dan1"
